from odoo import api, fields, models
from odoo.osv import expression


class AccPriorityGroupCounter(models.Model):
    _name = "acc.priority.group.counter"
    _description = "Clé de répartition par groupes de priorité"
    _rec_name = "acc_counter_id"

    # ------------------------------------------------------
    # Fields declaration
    # ------------------------------------------------------

    acc_priority_group_id = fields.Many2one(
        "acc.priority.group",
        "Groupe de priorité",
        required=True,
        ondelete="cascade",
        group_expand="_group_expand_acc_priority_group_id",
    )
    acc_operation_id = fields.Many2one(
        related="acc_priority_group_id.acc_operation_id", store=True, readonly=True
    )
    acc_counter_id = fields.Many2one(
        "acc.counter",
        "PRM",
        required=True,
        ondelete="cascade",
        domain="[('acc_operation_id', '=', acc_operation_id),"
        "('type', 'in', ['del', 'del_inj'])]",
    )
    counter_street = fields.Char(
        related="acc_counter_id.street", store=True, readonly=True
    )
    counter_owner = fields.Char(
        related="acc_counter_id.partner_id.name", store=True, readonly=True
    )
    acc_counter_id_domain = fields.Binary(
        string="Counter domain", compute="_compute_acc_counter_id_domain"
    )

    acc_counter_share = fields.Float(string='Quote-part', help="Part de la production affectée à ce PRM, au prorata de l'ensemble des parts affectées aux autres PRMs")
    group_type_algo = fields.Selection(related="acc_priority_group_id.type_algo", readonly=True)
    acc_counter_percentage = fields.Float(compute="_compute_acc_counter_percentage", store=False)

    # ------------------------------------------------------
    # SQL Constraints
    # ------------------------------------------------------

    # ------------------------------------------------------
    # Default methods
    # ------------------------------------------------------

    # ------------------------------------------------------
    # Computed fields / Search Fields
    # ------------------------------------------------------

    @api.depends('acc_priority_group_id.acc_priority_group_counter_ids', "acc_counter_share")
    def _compute_acc_counter_percentage(self):
        for record in self:
            total_share = record.acc_priority_group_id.total_share
            record.acc_counter_percentage = record.acc_counter_share / total_share if total_share else 0

    @api.depends("acc_counter_id", "acc_priority_group_id")
    def _compute_acc_counter_id_domain(self):
        for counter in self:
            domain = [
                ("acc_operation_id", "=", counter.acc_operation_id.id),
                ("type", "in", ["del", "del_inj"]),
            ]
            extended_domain = expression.AND(
                [
                    domain,
                    [
                        (
                            "id",
                            "not in",
                            counter.acc_priority_group_id.acc_priority_group_counter_ids.acc_counter_id.ids,
                        )
                    ],
                ]
            )
            counter.acc_counter_id_domain = extended_domain

    # ------------------------------------------------------
    # Onchange / Constraints
    # ------------------------------------------------------
    # @api.constrains("acc_counter_id_domain", "acc_counter_id")
    # def _check_only_one_counter_per_group(self):
    #     """
    #     Vérification qu'il n'y a pas plusieurs fois le meme compteur dans un groupe
    #     """
    #
    #     domain = expression.AND([self.acc_counter_id_domain, ])
    #
    #     s = self.env["acc.counter"].search(self.acc_counter_id_domain)
    #     l = s.ids
    #     c = self.acc_counter_id.id
    #
    #     if c not in l:
    #         raise UserError("Compteur déjà affecté a ce groupe")

    # ------------------------------------------------------
    # CRUD methods (ORM overrides)
    # ------------------------------------------------------

    # ------------------------------------------------------
    # Actions
    # ------------------------------------------------------

    def _group_expand_acc_priority_group_id(self, groups, domain, order):
        operation_id = self.env.context.get("default_acc_operation_id")
        if not operation_id:
            group_ids = self.env["acc.priority.group"].search(
                domain=[], order="sequence, id"
            )
        else:
            group_ids = self.env["acc.priority.group"].search(
                domain=[("acc_operation_id", "=", operation_id)], order="sequence, id"
            )
        return group_ids

    def action_open_counter_share_form(self):
        print('action_open_counter_share_form')
        return {
            'type': 'ir.actions.act_window',
            'name': 'Modifier la quote-part du point de soutirage',
            'res_model': 'acc.priority.group.counter',
            'view_mode': 'form',
            'target': 'new',
            'res_id': self.id,
            'view_id': self.env.ref('oacc_repartition_keys.acc_operation_priority_group_counter_share_form').id,
            #'context': {'default_field_name': 'Auto-filled Text'}
        }
